#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>

void die(char *msg) {
	fprintf(stderr, "envdir: %s\n", msg);
	exit(111);
}

char *buf = NULL;
int bufsize = 0;

void size_buf(int size) {
	if (size <= bufsize)
		return;

	buf = realloc(buf, size);
	if (!buf)
		die("out of memory");
	bufsize = size;
}

int main(int argc, char **argv) {
	DIR *dir;

	if (argc < 3)
		die("usage: envdir dir command ...");

	dir = opendir(argv[1]);
	if (!dir)
		die("unable to open directory");

	while (1) {
		int fd;
		struct stat st;
		struct dirent *e = readdir(dir);

		if (!e)
			break;

		if (strcmp(e->d_name, ".") == 0
			|| strcmp(e->d_name, "..") == 0)
			continue;

	 	size_buf(strlen(argv[1]) + 1 + strlen(e->d_name) + 1);
		snprintf(buf, bufsize, "%s/%s", argv[1], e->d_name);

		fd = open(buf, O_RDONLY);
		if (fd < 0)
			die("unable to open file");
		if (fstat(fd, &st) < 0)
			die("unable to stat file");

		if (st.st_size == 0) {
			unsetenv(e->d_name);
		} else {
			int pos = 0, len = 0;
			char *p;
#define BLOCK 4096
			/* We leave an extra 1 so we can write a \n in
			   if it doesn't have one. */
			size_buf(BLOCK + 1);
			while (1) {
				int count = read(fd, buf + pos, BLOCK);
				if (count < 0)
					die("read failed");
				len += count;
				pos += count;
				if (count < BLOCK)
					break;
				size_buf(bufsize + BLOCK);
			}
			
			/* Find the newline, then trim back any spaces
			   and tabs. */
			buf[pos] = '\n';
			p = memchr(buf, '\n', len);
			while (p > buf
				&& (*p == ' ' || *p == '\t' || *p == '\n'))
				p--;
			p++;

			/* Write a terminating \0, then go back down
			   replacing \0 with \n. */ 
			*p = '\0';
			while (--p >= buf) {
				if (*p == '\0')
					*p = '\n';
			}

			if (setenv(e->d_name, buf, 1) < 0)
				die("unable to set variable");
		}
		close(fd);

	}

	execvp(argv[2], &argv[2]);
	die("unable to exec");

	return 0; /* NOTREACHED */
}

