#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

void die(char *msg) {
	fprintf(stderr, "svc: %s\n", msg);
	exit(111);
}

int main(int argc, char **argv) {
	/* These are sorted by the order in which they should be sent to
	   supervise when multiple options are given. */
	char *cmds = "xXudopchaitk";
	char *used = strdup(cmds), *p, *q;
	int fd;

	if (!used)
		die("out of memory");

	while (1) {
		char c = getopt(argc, argv, cmds);

		if (c == -1)
			break;
		for (p = cmds, q = used; ; p++, q++) {
			if (!*p)
				die("bad option");
			if (*p == c) {
				*q = '\0';
				break;
			}
		}
	}

	if ((argc - optind) < 1)
		die("usage: svc -[udopchaitkxX] service");

	if (chdir(argv[optind]) < 0)
		die("unable to chdir to service dir");

	fd = open("supervise/control", O_WRONLY | O_NONBLOCK);
	if (fd < 0) {
		if (errno == ENXIO)
			die("no supervise running for that service");
		die("unable to open control pipe");
	}
	for (p = cmds, q = used; *p; p++, q++) {
		if (!*q) {
			if (write(fd, p, 1) < 0)
				die("unable to write to control pipe");
		}
	}
	close(fd);

	return 0;
}

